@@ -100,6 +100,7 @@ PlEx_fail(qid_t qid)
     throw ex2;
   } else
   { // TODO: get the name of the PL_...() function that caused the problem:
+    // assert(0);
     throw PlUnknownError("False return code without exception");
   }
 }
@@ -351,6 +352,19 @@ PlTerm::copy_term_ref() const
 { return PlTerm(Plx_copy_term_ref(unwrap()));
 }
 
+_SWI_CPP2_CPP_inline
+void
+PlTerm::free_term_ref()
+{ if ( not_null() )
+    Plx_free_term_ref(unwrap());
+}
+
+_SWI_CPP2_CPP_inline
+void
+PlTerm::free_term_ref_reset()
+{ free_term_ref();
+  reset();
+}
 
 _SWI_CPP2_CPP_inline
 void
