@@ -179,15 +179,15 @@ public:
 
   friend C_t *PlUnwrapAsPtr(WrappedC<C_t>* obj) { return obj ? obj->unwrap_as_ptr() : nullptr; }
 
-  explicit WrappedC<C_t>(C_t v)
+  explicit WrappedC(C_t v)
     : C_(v) { }
 
-  WrappedC<C_t>(            const WrappedC<C_t>&) = default;
-  WrappedC<C_t>& operator =(const WrappedC<C_t>&) = default;
+  WrappedC(            const WrappedC<C_t>&) = default;
+  WrappedC& operator =(const WrappedC<C_t>&) = default;
   // This simple wrapper class doesn't need a move constructor or
   // move operator =.
 
-  ~WrappedC<C_t>() { }
+  ~WrappedC() { }
 
   operator bool() const = delete; // Use not_null(), is_null() instead
   bool operator ==(WrappedC<C_t> o) const { return C_ == o.C_; }
@@ -316,15 +316,15 @@ public:
   bool operator ==(PlAtom to) const { return unwrap() == to.unwrap(); }
   bool operator !=(PlAtom to) const { return unwrap() != to.unwrap(); }
 
-  [[deprecated("use as_string() or ==PlAtom")]]  bool operator ==(const char *s)         const { return eq(s); }
-  [[deprecated("use as_string() or ==PlAtom")]]  bool operator ==(const wchar_t *s)      const { return eq(s); }
-  [[deprecated("use as_string() or ==PlAtom")]]  bool operator ==(const std::string& s)  const { return eq(s); }
-  [[deprecated("use as_string() or ==PlAtom")]]  bool operator ==(const std::wstring& s) const { return eq(s); }
-  [[deprecated("use PlAtom instead of atom_t")]] bool operator ==(atom_t to)             const { return unwrap() == to; }
+  [[deprecated("use as_string() and std::string::operator==() or ==PlAtom")]]   bool operator ==(const char *s)         const { return eq(s); }
+  [[deprecated("use as_string() and std::string::operator==() or ==PlAtom")]]   bool operator ==(const wchar_t *s)      const { return eq(s); }
+  [[deprecated("use as_string() and std::string::operator==() or ==PlAtom")]]   bool operator ==(const std::string& s)  const { return eq(s); }
+  [[deprecated("use as_swtring() and std::wstring::operator==() or ==PlAtom")]] bool operator ==(const std::wstring& s) const { return eq(s); }
+  [[deprecated("use PlAtom instead of atom_t")]]                                bool operator ==(atom_t to)             const { return unwrap() == to; }
 
-  [[deprecated("use as_string() or !=PlAtom")]]  bool operator !=(const char *s)         const { return !eq(s); }
-  [[deprecated("use as_string() or !=PlAtom")]]  bool operator !=(const wchar_t *s)      const { return !eq(s); }
-  [[deprecated("use PlAtom instead of atom_t")]] bool operator !=(atom_t to)             const { return unwrap() != to; }
+  [[deprecated("use as_string() and std::string::operator!=() or !=PlAtom")]]   bool operator !=(const char *s)         const { return !eq(s); }
+  [[deprecated("use as_string() and std::string::operator!=() or !=PlAtom")]]   bool operator !=(const wchar_t *s)      const { return !eq(s); }
+  [[deprecated("use PlAtom instead of atom_t")]]                                bool operator !=(atom_t to)             const { return unwrap() != to; }
 
   // TODO: when C++17 becomes standard, rename register_ref() to register().
 
@@ -556,26 +556,32 @@ public:
   void put_chars(int flags, size_t len, const char *chars) { Plx_put_chars(unwrap(), flags, len, chars); }
   void put_list_chars(const char *chars)                   { Plx_put_list_chars(unwrap(), chars); }
   void put_list_codes(const char *chars)                   { Plx_put_list_codes(unwrap(), chars); }
+  // TODO: add std::string versions of the following
   void put_atom_nchars(size_t l, const char *chars)        { Plx_put_atom_nchars(unwrap(), l, chars); }
   void put_string_nchars(size_t len, const char *chars)    { Plx_put_string_nchars(unwrap(), len, chars); }
   void put_list_nchars(size_t l, const char *chars)        { Plx_put_list_nchars(unwrap(), l, chars); }
   void put_list_ncodes(size_t l, const char *chars)        { Plx_put_list_ncodes(unwrap(), l, chars); }
   void put_integer(long i)                                 { Plx_put_integer(unwrap(), i); }
+  void put_int64(int64_t i)                                { Plx_put_int64(unwrap(), i); }
+  void put_uint64(uint64_t i)                              { Plx_put_uint64(unwrap(), i); }
   void put_pointer(void *ptr)                              { Plx_put_pointer(unwrap(), ptr); }
   void put_float(double f)                                 { Plx_put_float(unwrap(), f); }
   void put_functor(PlFunctor functor)                      { Plx_put_functor(unwrap(), functor.unwrap()); }
   void put_list()                                          { Plx_put_list(unwrap()); }
   void put_nil()                                           { Plx_put_nil(unwrap()); }
   void put_term(PlTerm t2)                                 { Plx_put_term(unwrap(), t2.unwrap()); }
+  void put_blob( void *blob, size_t len, PL_blob_t *type)  { Plx_put_blob(unwrap(), blob, len, type); }
+  PlRecord record() const;
+
+  // TODO: When the following are implemented, add them
+  //       to the deleted methods in PlTermScoped:
   // TODO: PL_put_dict(term_t t, atom_t tag, size_t len, const atom_t *keys, term_t values)
   // TODO: PL_cons_functor(term_t h, functor_t f, ...)
   // TODO: PL_cons_functor_v(term_t h, functor_t fd, term_t a0)
   // TODO: PL_cons_list(term_t l, term_t h, term_t t)
-  void put_blob( void *blob, size_t len, PL_blob_t *type)  { Plx_put_blob(unwrap(), blob, len, type); }
 
   // TODO: PL_unify_*()?
   // TODO: PL_skip_list()
-  PlRecord record() const;
 
 					/* PlTerm --> C */
   [[deprecated("use as_long()")]]     explicit operator long()     const { return as_long(); }
@@ -640,6 +646,8 @@ public:
   PlAtom name() const; // throws PlTypeError if not a "compound" or atom
   [[nodiscard]] bool name_arity(PlAtom *name, size_t *arity) const; // name and/or arity can be nullptr
   [[nodiscard]] PlTerm copy_term_ref() const;
+  void free_term_ref();
+  void free_term_ref_reset();
 
   // The assignment operators from version 1 have been removed because
   // of possible confusion with the standard assignment and copy
@@ -710,25 +718,25 @@ public:
   bool operator <=(PlTerm t2) const { return compare(t2) <= 0; }
   bool operator >=(PlTerm t2) const { return compare(t2) >= 0; }
 					/* comparison (long) */
-  [[deprecated("use as_int64_t()")]] bool operator ==(int64_t v) const;
-  [[deprecated("use as_int64_t()")]] bool operator !=(int64_t v) const;
-  [[deprecated("use as_int64_t()")]] bool operator < (int64_t v) const;
-  [[deprecated("use as_int64_t()")]] bool operator > (int64_t v) const;
-  [[deprecated("use as_int64_t()")]] bool operator <=(int64_t v) const;
-  [[deprecated("use as_int64_t()")]] bool operator >=(int64_t v) const;
+  [[deprecated("use as_int64_t() and int64_t::operator==()")]] bool operator ==(int64_t v) const;
+  [[deprecated("use as_int64_t() and int64_t::operator!=()")]] bool operator !=(int64_t v) const;
+  [[deprecated("use as_int64_t() and int64_t::operator<()")]]  bool operator < (int64_t v) const;
+  [[deprecated("use as_int64_t() and int64_t::operator>()")]]  bool operator > (int64_t v) const;
+  [[deprecated("use as_int64_t() and int64_t::operator<=()")]] bool operator <=(int64_t v) const;
+  [[deprecated("use as_int64_t() and int64_t::operator>=()")]] bool operator >=(int64_t v) const;
 
 					/* comparison (atom, string) */
-  [[deprecated("use as_string()")]]  bool operator ==(const char *s)         const { return eq(s); }
-  [[deprecated("use as_string()")]]  bool operator ==(const wchar_t *s)      const { return eq(s); }
-  [[deprecated("use as_wstring()")]] bool operator ==(const std::string& s)  const { return eq(s); }
-  [[deprecated("use as_wstring()")]] bool operator ==(const std::wstring& s) const { return eq(s); }
-  [[deprecated("use as_atom()")]]    bool operator ==(PlAtom a)              const { return eq(a); }
+  [[deprecated("use as_string() and std::string::operator==()")]]    bool operator ==(const char *s)         const { return eq(s); }
+  [[deprecated("use as_string() and std::string::operator==()")]]    bool operator ==(const wchar_t *s)      const { return eq(s); }
+  [[deprecated("use as_wstring() and std::string::operator==()")]]   bool operator ==(const std::string& s)  const { return eq(s); }
+  [[deprecated("use as_wstring() and std::wstring::operator==()")]]  bool operator ==(const std::wstring& s) const { return eq(s); }
+  [[deprecated("use as_atom()")]]                                    bool operator ==(PlAtom a)              const { return eq(a); }
 
-  [[deprecated("use as_string()")]]  bool operator !=(const char *s)         const { return !eq(s); }
-  [[deprecated("use as_wstring()")]] bool operator !=(const wchar_t *s)      const { return !eq(s); }
-  [[deprecated("use as_string()")]]  bool operator !=(const std::string& s)  const { return !eq(s); }
-  [[deprecated("use as_wstring()")]] bool operator !=(const std::wstring& s) const { return !eq(s); }
-  [[deprecated("use as_atom()")]]    bool operator !=(PlAtom a)              const { return !eq(a); }
+  [[deprecated("use as_string() and std::string::operator==()")]]    bool operator !=(const char *s)         const { return !eq(s); }
+  [[deprecated("use as_wstring() and std::wstring::operator==()")]]  bool operator !=(const wchar_t *s)      const { return !eq(s); }
+  [[deprecated("use as_string() and std::string::operator==()")]]    bool operator !=(const std::string& s)  const { return !eq(s); }
+  [[deprecated("use as_wstring() and std::wstring::operator==()")]]  bool operator !=(const std::wstring& s) const { return !eq(s); }
+  [[deprecated("use as_atom()")]]                                    bool operator !=(PlAtom a)              const { return !eq(a); }
 
   // E.g.: t.write(Serror, 1200, PL_WRT_QUOTED);
   [[nodiscard]] int write(IOSTREAM *s, int precedence, int flags) const
@@ -753,6 +761,107 @@ private:
 };
 
 
+// PlTermScoped is an *experimental* inteface, which may change
+// in the future. It implements a PlTerm that is automatically
+// freed when it goes out of scope. The API is similar to
+// std::unique_ptr.
+
+class PlTermScoped : public PlTerm
+{
+public:
+  explicit PlTermScoped()
+  : PlTerm()
+  { }
+
+  explicit PlTermScoped(PlTerm t)
+    : PlTerm(t.copy_term_ref())
+  { }
+
+  explicit PlTermScoped(term_t t)
+    : PlTerm(Plx_copy_term_ref(t))
+  { }
+
+  explicit PlTermScoped(PlTermScoped&& moving) noexcept
+    : PlTerm(moving)
+  { moving.reset();
+  }
+
+  PlTermScoped& operator=(PlTermScoped&& moving) noexcept
+  { if ( this != &moving )
+    { reset(moving);
+      moving.reset();
+    }
+    return *this;
+  }
+
+  PlTermScoped(const PlTermScoped&) = delete;
+  PlTermScoped& operator=(PlTermScoped const&) = delete;
+
+  ~PlTermScoped()
+  { free_term_ref_reset(); // TODO: reset() isn't needed?
+  }
+
+  void reset()
+  { free_term_ref();
+    PlTerm::reset();
+  }
+
+  void reset(PlTerm src)
+  { free_term_ref();
+    PlTerm::reset(src);
+  }
+
+  PlTerm get() const noexcept { return PlTerm(unwrap()); }
+
+  PlTermScoped release() noexcept
+  { term_t t = unwrap();
+    reset();
+    return PlTermScoped(t);
+  }
+
+  void swap(PlTermScoped& src) noexcept
+  { // std::swap(*this, src);
+    term_t this_unwrap = unwrap();
+    // Must use PlTerm::reset() to avoid call to PL_free_term_ref():
+    static_cast<PlTerm*>(this)->reset(PlTerm(src.unwrap()));
+    static_cast<PlTerm>(src).reset(PlTerm(this_unwrap));
+  }
+
+  // The put_*() and cons_*() methods are incompatible with the
+  // call to PL_free_term_ref() in the destructor.
+  void put_variable() = delete;
+  void put_atom(PlAtom a) = delete;
+  void put_bool(int val) = delete;
+  void put_atom_chars(const char *chars) = delete;
+  void put_string_chars(const char *chars) = delete;
+  void put_chars(int flags, size_t len, const char *chars) = delete;
+  void put_list_chars(const char *chars) = delete;
+  void put_list_codes(const char *chars) = delete;
+  void put_atom_nchars(size_t l, const char *chars) = delete;
+  void put_string_nchars(size_t len, const char *chars) = delete;
+  void put_list_nchars(size_t l, const char *chars) = delete;
+  void put_list_ncodes(size_t l, const char *chars) = delete;
+  void put_integer(long i) = delete;
+  void put_pointer(void *ptr) = delete;
+  void put_float(double f) = delete;
+  void put_functor(PlFunctor functor) = delete;
+  void put_list() = delete;
+  void put_nil() = delete;
+  void put_term(PlTerm t2) = delete;
+  void put_blob( void *blob, size_t len, PL_blob_t *type) = delete;
+};
+
+// TODO: verify that this is the right way to specialize
+//       std::swap()'s use with std containers:
+namespace std
+{
+  inline
+  void swap(PlTermScoped& lhs, PlTermScoped& rhs) noexcept
+  { lhs.swap(rhs);
+  }
+}
+
+
 class PlTerm_atom : public PlTerm
 {
 public:
@@ -858,7 +967,7 @@ private:
 public:
   explicit PlTermv(size_t n = 0)
     : size_(n),
-      a0_(n ? Plx_new_term_refs(static_cast<int>(n)) : PlTerm::null)
+      a0_(n ? Plx_new_term_refs(n) : PlTerm::null)
   { if ( size_  )
       PlEx<bool>(a0_ != (term_t)0);
   }
@@ -947,7 +1056,7 @@ public:
 class PlRecord : public WrappedC<record_t>
 {
 public:
-  PlRecord(PlTerm t)
+  explicit PlRecord(PlTerm t)
     : WrappedC<record_t>(Plx_record(t.unwrap()))
   { }
 
@@ -959,12 +1068,25 @@ public:
     : WrappedC<record_t>(r)
   { }
 
-  PlRecord& operator =(const PlRecord& r)
+  PlRecord(PlRecord&& r) noexcept
+    : WrappedC<record_t>(r)
+  { r.reset();
+  }
+
+  PlRecord& operator=(const PlRecord& r)
   { if ( this != &r )
       reset(r);
     return *this;
   }
 
+  PlRecord& operator=(PlRecord&& r) noexcept
+  { if ( this != &r )
+    { reset(r);
+      r.reset();
+    }
+    return *this;
+  }
+
   PlTerm term() const
   { PlTerm_var t;
     Plx_recorded(unwrap(), t.unwrap());
@@ -982,7 +1104,7 @@ public:
   }
 
   ~PlRecord()
-  { // TODO: erase();
+  { // TODO: erase(); -- but probably doesn't work with move constructor
   }
 };
 
@@ -1017,6 +1139,14 @@ public:
     : C_(init(t))
   { }
 
+  PlRecordExternalCopy(const std::string& external)
+    : C_(external)
+  { }
+
+  PlRecordExternalCopy(const char*external, size_t len)
+    : C_(std::string(external, len))
+  { }
+
   PlRecordExternalCopy(const PlRecordExternalCopy& r) = default;
   PlRecordExternalCopy& operator =(const PlRecordExternalCopy&) = delete;
   ~PlRecordExternalCopy() = default;
@@ -1027,14 +1157,26 @@ public:
     return t;
   }
 
+  static PlTerm
+  term(const char* data)
+  { PlTerm_var t;
+    Plx_recorded_external(data, t.unwrap());
+    return t;
+  }
+
+  static PlTerm
+  term(const std::string& data)
+  { return term(data.data());
+  }
+
   const std::string& data() const { return C_; }
 
 private:
   std::string init(PlTerm t)
   { size_t len;
-    char *s =  Plx_record_external(t.unwrap(), &len);
-    std::string result(s, len);
-    Plx_erase_external(s);
+    char *external =  Plx_record_external(t.unwrap(), &len);
+    std::string result(external, len);
+    Plx_erase_external(external);
     return result;
   }
 
@@ -1260,7 +1402,7 @@ public:
 		 *	 CALLING PROLOG		*
 		 *******************************/
 
-class PlFrame : WrappedC<fid_t>
+class PlFrame : public WrappedC<fid_t>
 {
 public:
   PlFrame()
@@ -1294,7 +1436,7 @@ public:
 [[nodiscard]] bool PlRewindOnFail(std::function<bool()> f);
 
 
-class PlQuery : WrappedC<qid_t>
+class PlQuery : public WrappedC<qid_t>
 {
 public:
   PlQuery(PlPredicate pred, const PlTermv& av, int flags = PL_Q_PASS_EXCEPTION)
@@ -1668,12 +1810,12 @@ class PlForeignContextPtr
 {
 public:
   [[deprecated("Use PlControl::context_unique_ptr")]]
-  explicit PlForeignContextPtr<ContextType>(PlControl handle)
+  explicit PlForeignContextPtr(PlControl handle)
   { ptr_.reset(static_cast<ContextType *>(handle.foreign_context_address()));
   }
 
-  PlForeignContextPtr<ContextType>(const PlForeignContextPtr<ContextType>&) = delete;
-  PlForeignContextPtr<ContextType>(PlForeignContextPtr<ContextType>&&) = delete;
+  PlForeignContextPtr(const PlForeignContextPtr<ContextType>&) = delete;
+  PlForeignContextPtr(PlForeignContextPtr<ContextType>&&) = delete;
   PlForeignContextPtr<ContextType>& operator =(const PlForeignContextPtr<ContextType>&) = delete;
   PlForeignContextPtr<ContextType>& operator =(PlForeignContextPtr<ContextType>&&) = delete;
 
@@ -1720,7 +1862,7 @@ public:
     // outside of a PREDICATE.
     if ( ref && type == ref->blob_t_ )
     { if ( len != sizeof *ref )
-        PL_system_error("Invalid size %zd (should be %zd) for %s", len, sizeof *ref, typeid(C_t).name());
+        PL_api_error("Invalid size %zd (should be %zd) for %s", len, sizeof *ref, typeid(C_t).name());
       return ref;
     }
     return nullptr;
@@ -1733,7 +1875,7 @@ public:
     // Can't throw PlException here because might be in a context
     // outside of a PREDICATE.
     if ( !ref )
-      PL_system_error("Failed cast to %s", typeid(C_t).name());
+      PL_api_error("Failed cast to %s", typeid(C_t).name());
     return ref;
   }
 
@@ -1756,7 +1898,7 @@ public:
     }
     PREDICATE_CATCH(rc = false)
     if ( !rc )
-      PL_system_error("Failed acquire() for %s", typeid(C_t).name());
+      PL_api_error("Failed acquire() for %s", typeid(C_t).name());
     // TODO: if ( ! rc ) Plx_clear_exception() ?
   }
 
@@ -1794,7 +1936,7 @@ public:
     }
     PREDICATE_CATCH(rc_try = false; rc = 0;)
     if ( !rc_try )
-      PL_system_error("Failed compare() for %s", typeid(C_t).name());
+      PL_api_error("Failed compare() for %s", typeid(C_t).name());
     return rc;
   }
 
